#include <stdio.h>
#include <ctype.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include "util.c"

#define LL printf("%s:%d\n", __FILE__, __LINE__)

/* These are for parsing the input for data from the form */
#define MAX_ENTRIES 10

typedef struct {
    char *name;
    char *val;
} entry;

char *makeword(char *line, char stop);
char *fmakeword(FILE *f, char stop, int *len);
char x2c(char *what);
void unescape_url(char *url);
void plustospace(char *str);
/* End of form parsing requirements */

#define DATA_FILE "scores.dat"
#define MAX_SCORES 20
#define MAX_CHARS 100

typedef struct {
  char  Heading[MAX_CHARS] ;
  float MaxPossible ;
  float MaxScored ;
  float MinScored ;
  float MeanScore ;
  float MedianScore ;
  float StandardDev ;
  float YourScore ;
} SCORE ;

typedef struct {
  int TotalStudents ;
  int Rank ;
  int nScores ;
  SCORE Scores[MAX_SCORES] ;
  char Name[MAX_CHARS] ;
  long Id ;
} STUDENT_REC ;

typedef enum {
   INT_ERR, ILL_STUD, ALL_OK
} RET_TYPES ;

static RET_TYPES GetStudent (int argc, char **argv, STUDENT_REC *) ;
static long GetStudentId (int argc, char **argv) ;

int main (int argc, char **argv)
{
  STUDENT_REC Stud ;
  STUDENT_REC *stud = &Stud ;
  int i ;
  RET_TYPES rt ;

  /* First print out the junk stuff */
  printf ("Content-type: text/html\n\n") ;

  printf ("<title>Your scores</title>\n") ;
  printf ("<p>\n") ;
  printf ("<body bgcolor=#ffffff>\n</body>\n") ;
  printf ("<hr>\n<p>\n") ;
  printf ("<center><h2><font size=+3>Y</font>OUR <font size=+3>S</font>CORES</h2> </center>\n<p><hr><p>\n") ;

  /* Get the student id */
  rt = GetStudent (argc, argv, stud) ;

  switch (rt)
    {
    case ILL_STUD:
      printf ("Student Identifier <b>%ld</b> not found.\n", stud->Id) ;
      break ;
    case ALL_OK:
      printf ("<center>\n") ;
      /* printf ("<b>Name:</b> %s<br>\n", stud->Name) ; */
      printf ("<b>Student Id:</b> %ld<br>\n", stud->Id) ;
      printf ("<b>Present Rank:</b> %d out of %d students.<br><p>\n",
	      stud->Rank, stud->TotalStudents) ;
      printf ("<table border>\n") ;
      printf ("<tr valign=bottom>\n") ;
      printf ("<th>For </th> <th>Your <br> Score</th> <th> Max <br>\n") ;
      printf ("Possible </th> <th> Max <br> Scored </th>\n") ;
      printf ("<th> Min <br> Scored </th> <th> Mean </th> <th>Median</th>\n") ;
      printf ("<th> Standard <br> Deviation </th>\n") ;
      printf ("</tr>\n") ;
      for (i = 0; i < stud->nScores; i++)
	{
	  printf ("<tr align=center>\n") ;
	  printf ("<td>%s</td>\n", stud->Scores[i].Heading) ;
	  printf ("<th>%5.2f</th>\n", stud->Scores[i].YourScore) ;
	  printf ("<td>%5.2f</td>\n", stud->Scores[i].MaxPossible) ;
	  printf ("<td>%5.2f</td>\n", stud->Scores[i].MaxScored) ;
	  printf ("<td>%5.2f</td>\n", stud->Scores[i].MinScored) ;
	  printf ("<td>%5.2f</td>\n", stud->Scores[i].MeanScore) ;
	  printf ("<td>%5.2f</td>\n", stud->Scores[i].MedianScore) ;
	  printf ("<td>%5.2f</td>\n", stud->Scores[i].StandardDev) ;
	  printf ("</tr>\n") ;
	}
      printf ("</table>\n") ;
      printf ("</center>\n") ;
      printf ("<p>\n<hr>\n<p>If you notice a mistake in your scores here, please contact <a href=mailto:dahlen@cs.umn.edu>dahlen@cs.umn.edu</a> immediately.\n") ;
      break ;
    case INT_ERR: /* Fall Thru */
    default:
      printf ("An internal error occured. Please contact <a href=mailto:dahlen@cs.umn.edu>dahlen@cs.umn.edu</a>\n") ;
      break ;
    }


  printf ("<p>\n<hr>\n<p>\n<address><a href=http://www.cs.umn.edu/~dahlen>Brent Dahlen </a>\n") ;
  printf ("<address><a href=http://www.cs.umn.edu/~doyle> Dave Doyle</a>\n") ;
}

static RET_TYPES GetStudent (int argc, char **argv, STUDENT_REC *stud)
{
  SCORE *score ;
  FILE *fp ;
  char Buff[1000], Buff1[1000], StatString[1000] ;
  long id, IdInFile ;
  int i, rc ;
  char *ptr ;

  /* Initialize some variables */
  stud->nScores = 0 ;
  score = &stud->Scores[stud->nScores] ;

  /* Find which Id is being asked */
  id = GetStudentId (argc, argv) ;
  if (id == -1)
    return (INT_ERR) ;

  /* Open the data file */
  fp = fopen (DATA_FILE, "r") ;
  if (fp == NULL)
    return (INT_ERR) ;

  /* First let us fill up the statistics part */
  while (fgets (Buff, sizeof (Buff), fp) != NULL)
    {
      StatString[0] = 0 ;

      /* check if it is the statistics string */
      sscanf (Buff, "%s", StatString) ;
      if (strcmp (StatString, "statistics") == 0)
	{
	  /* We have a statistics on our hand */

	  /* This string looks like:
	   * statistics for XXXX base value is: NNN
	   */
	  sscanf (Buff, "%*s %*s %s %*[^:]: %f",
		  score->Heading, &score->MaxPossible) ;

	  /* Get the next line: zeroes: NNN max: NNN */
	  if (fgets (Buff, sizeof (Buff), fp) == NULL)
	    return (INT_ERR) ;
	  sscanf (Buff, "%*[^:]: %*[^:]: %f", &score->MaxScored) ;

	  /* Get the next line: omitted: NNN min: NNN*/
	  if (fgets (Buff, sizeof (Buff), fp) == NULL)
	    return (INT_ERR) ;
	  sscanf (Buff, "%*[^:]: %*[^:]: %f", &score->MinScored) ;

	  /* Get the next line: incomplete: NNN mean: NNN*/
	  if (fgets (Buff, sizeof (Buff), fp) == NULL)
	    return (INT_ERR) ;
	  sscanf (Buff, "%*[^:]: %*[^:]: %f", &score->MeanScore) ;

	  /* Get the next line: other grades: NNN median: NNN*/
	  if (fgets (Buff, sizeof (Buff), fp) == NULL)
	    return (INT_ERR) ;
	  sscanf (Buff, "%*[^:]: %*[^:]: %f", &score->MedianScore) ;

	  /* Get the next line: total grades: NNN std. dev.: NNN*/
	  if (fgets (Buff, sizeof (Buff), fp) == NULL)
	    return (INT_ERR) ;
	  sscanf (Buff, "%*[^:]: %d %*[^:]: %f",
		  &stud->TotalStudents, &score->StandardDev) ;

	  /* Now on to the next one */
	  stud->nScores++ ;

	  /* Are we done with the headers ? */
          if (strcmp (score->Heading, "total") == 0)
	    break ;

	  score = &stud->Scores[stud->nScores] ;
	}
    }

  /* Now we have to look for the scores */
  while (fgets (Buff, sizeof(Buff), fp) != NULL)
    {
      if ((rc = sscanf (Buff, "%*[^0-9] %ld", &IdInFile)) < 1)
	continue ;

      strcpy (Buff1, Buff) ;
      if (IdInFile == id)
	{
	  ptr = strtok (Buff, " ") ;
	  while (ptr != NULL)
	    {
	      if (isdigit (*ptr))
		{
		  if (atol (ptr) != id)
		    return (INT_ERR) ;
		  for (i = 0; i < stud->nScores; i++)
		    {
		      ptr = strtok (NULL, " ") ;
		      if (isdigit (*ptr))
			{
			  stud->Scores[i].YourScore = atof (ptr) ;
			}
		      else
			return (INT_ERR) ;
		    }
		  /* get the rank out */
		  ptr = strtok (NULL, " ") ;
		  if (isdigit (*ptr))
		    {
		      stud->Rank = atoi (ptr) ;
		    }
		  else
		    return (INT_ERR) ;
		  stud->Id = id ;
		  sscanf (Buff1, "%[^0-9]", stud->Name) ;
		  return (ALL_OK) ;
		}
	      else
		/* Something else, ignore */
		ptr = strtok (NULL, " ") ;
	    }
	}
    }
  /* Score not found */
  stud->Id = id ;
  return (ILL_STUD) ;
}

static long GetStudentId (int argc, char **argv)
{
#define MAX_ENTRIES 10

  typedef struct {
    char *name;
    char *val;
  } entry;

  extern char *makeword(char *line, char stop);
  extern char *fmakeword(FILE *f, char stop, int *len);
  extern char x2c(char *what);
  extern void unescape_url(char *url);
  extern void plustospace(char *str);

  entry entries[MAX_ENTRIES] ;
  register int x, m = 0 ;
  int cl ;

  /* Get the form data */
  cl = atoi(getenv("CONTENT_LENGTH"));

  for(x=0;cl && (!feof(stdin));x++) {
    m=x;
    entries[x].val = fmakeword(stdin,'&',&cl);
    plustospace(entries[x].val);
    unescape_url(entries[x].val);
    entries[x].name = makeword(entries[x].val,'=');
  }
  /* Getting form data over */  

/*  printf ("number is: %ld\n", atol (entries[0].val)) ; */
  return (atol(entries[0].val)) ;
}
